<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:lav="http://software.in2p3.fr/lavoisier/config.xsd"
                xmlns="http://software.in2p3.fr/lavoisier/config.xsd"
                xmlns:xpath="xalan://fr.in2p3.lavoisier.helpers.xpath"
                exclude-result-prefixes="lav xpath">
    <xsl:output method="xml" indent="yes"/>
    <xsl:variable name="processors" select="document('processors.xml')/properties"/>

    <xsl:template match="/">
        <xsl:apply-templates/>
    </xsl:template>

    <xsl:template match="lav:processor">
        <xsl:copy><xsl:copy-of select="@*"/>
            <xsl:choose>
                <xsl:when test="starts-with(normalize-space(@match), '/')">
                    <match>
                        <xsl:copy-of select="xpath:parse(@match, ancestor::lav:view/@name)"/>
                    </match>
                </xsl:when>
                <xsl:when test="starts-with(normalize-space(@match), 'eval(')"/>
                <xsl:otherwise>
                    <xsl:message terminate="yes">
                        Attribute @match must be either an absolute path or the eval() function: <xsl:value-of select="@match"/>
                    </xsl:message>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="lav:processor/lav:parameter">
        <xsl:copy><xsl:copy-of select="@*"/>
            <xsl:variable name="parameters" select="$processors/entry[@key=current()/parent::lav:processor/@type]/text()"/>
            <xsl:choose>
                <!-- XPath expression parameters -->
                <xsl:when test="contains($parameters, concat('|',@name,'|'))">
                    <xsl:choose>
                        <xsl:when test="@eval">
                            <xsl:copy-of select="xpath:parse(@eval, ancestor::lav:view/@name)"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:copy-of select="xpath:parse(concat('&quot;',text(),'&quot;'), ancestor::lav:view/@name)"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:when>
                <!-- absolute/relative XPath parameters -->
                <xsl:when test="contains($parameters, concat('/',@name,'/'))">
                    <xsl:choose>
                        <xsl:when test="@eval">
                            <xsl:copy-of select="xpath:parse(@eval, ancestor::lav:view/@name)"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:message terminate="yes">This parameter must be set in attribute eval:
                                <xsl:text>/cfg:config</xsl:text>
                                <xsl:for-each select="ancestor::lav:view">/cfg:view[@name='<xsl:value-of select="@name"/>']</xsl:for-each>
                                <xsl:text>/cfg:processors</xsl:text>
                                <xsl:for-each select="parent::lav:processor">/cfg:processor[@type='<xsl:value-of select="@type"/>']</xsl:for-each>
                                <xsl:for-each select="self::lav:parameter">/cfg:parameter[@name='<xsl:value-of select="@name"/>']</xsl:for-each>
                            </xsl:message>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:when>
                <!-- other parameters -->
                <xsl:otherwise>
                    <xsl:apply-templates/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:copy>
    </xsl:template>

    <!-- default rules -->
    <xsl:template match="@*">
        <xsl:copy-of select="."/>
    </xsl:template>
    <xsl:template match="comment()">
        <xsl:comment><xsl:value-of select="."/></xsl:comment>
    </xsl:template>
    <xsl:template match="*">
        <xsl:copy>
            <xsl:apply-templates select="@*|*|text()|comment()"/>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet>