<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:x="http://www.w3.org/TR/xpath">
    <xsl:output method="text"/>

    <xsl:template match="x:NameStep">
        <xsl:param name="reverse">false</xsl:param>
        <xsl:if test="position()>1 or parent::x:RelativeLocationPath/preceding-sibling::x:FilterExpr or parent::x:AbsoluteLocationPath">/</xsl:if>
        <xsl:choose>
            <xsl:when test="$reverse='false'"><xsl:value-of select="@axis"/></xsl:when>
            <xsl:when test="following-sibling::x:NameStep[1]/@axis='attribute'">parent</xsl:when>
            <xsl:when test="following-sibling::x:NameStep[1]/@axis='child'">parent</xsl:when>
            <xsl:when test="following-sibling::x:NameStep[1]/@axis='descendant'">ancestor</xsl:when>
            <xsl:when test="following-sibling::x:NameStep[1]/@axis='descendant-or-self'">ancestor-or-self</xsl:when>
            <xsl:when test="following-sibling::x:NameStep[1]/@axis='self'">self</xsl:when>
            <xsl:when test="following-sibling::x:TextNodeStep[1]">parent</xsl:when>
            <xsl:when test="following-sibling::x:CommentNodeStep[1]">parent</xsl:when>
            <xsl:otherwise>
                <xsl:message terminate="yes">INTERNAL ERROR (unexpected node): <xsl:value-of select="local-name()"/></xsl:message>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:text/>::<xsl:if test="@prefix"><xsl:value-of select="@prefix"/>:</xsl:if><xsl:value-of select="@localName"/>
        <xsl:apply-templates select="x:Predicate"/>
    </xsl:template>
    <xsl:template match="x:TextNodeStep">
        <xsl:if test="position()>1 or parent::x:RelativeLocationPath/preceding-sibling::x:FilterExpr">/</xsl:if>
        <xsl:text>text()</xsl:text>
        <xsl:apply-templates select="x:Predicate"/>
    </xsl:template>
    <xsl:template match="x:CommentNodeStep">
        <xsl:if test="position()>1 or parent::x:RelativeLocationPath/preceding-sibling::x:FilterExpr">/</xsl:if>
        <xsl:text>comment()</xsl:text>
        <xsl:apply-templates select="x:Predicate"/>
    </xsl:template>
    <xsl:template match="x:AllNodeStep">
        <xsl:if test="position()>1 or parent::x:RelativeLocationPath/preceding-sibling::x:FilterExpr or parent::x:AbsoluteLocationPath">/</xsl:if>
        <xsl:choose>
            <xsl:when test="@axis='self'">.</xsl:when>
            <xsl:otherwise><xsl:value-of select="@axis"/>::*<xsl:text/></xsl:otherwise>
        </xsl:choose>
        <xsl:apply-templates select="x:Predicate"/>
    </xsl:template>
    <xsl:template match="x:ProcessingInstructionNodeStep">&lt;?<xsl:value-of select="@s"/>?&gt;</xsl:template>

    <xsl:template match="x:Function">
        <xsl:apply-templates select="." mode="serialize"/>
    </xsl:template>
    <xsl:template match="x:Function" mode="serialize">
        <xsl:if test="@s"><xsl:value-of select="@s"/>:</xsl:if>
        <xsl:value-of select="@s1"/>(<xsl:for-each select="
                (x:PathExpr|x:UnaryExpr|x:UnionExpr|x:OrExpr|x:AndExpr)[not(following-sibling::x:*[1][not(contains('PathExpr|UnaryExpr|UnionExpr|OrExpr|AndExpr',local-name()))])]
              | *[not(contains('PathExpr|UnaryExpr|UnionExpr|OrExpr|AndExpr',local-name()))][following-sibling::x:*[1][contains('PathExpr|UnaryExpr|UnionExpr|OrExpr|AndExpr',local-name())] or position()=last()]
        ">
            <xsl:if test="position()>1">,</xsl:if><xsl:apply-templates select="."/>
        </xsl:for-each>)<xsl:text/>
    </xsl:template>
    <xsl:template match="x:UnionExpr | x:OrExpr | x:AndExpr">
        <xsl:for-each select="
                (x:PathExpr|x:UnaryExpr|x:UnionExpr|x:OrExpr|x:AndExpr)[not(following-sibling::x:*[1][not(contains('PathExpr|UnaryExpr|UnionExpr|OrExpr|AndExpr',local-name()))])]
              | *[not(contains('PathExpr|UnaryExpr|UnionExpr|OrExpr|AndExpr',local-name()))][following-sibling::x:*[1][contains('PathExpr|UnaryExpr|UnionExpr|OrExpr|AndExpr',local-name())] or position()=last()]
        ">
            <xsl:if test="position()>1"><xsl:value-of select="parent::x:*/@symbol"/></xsl:if><xsl:apply-templates select="."/>
        </xsl:for-each>
    </xsl:template>
    <xsl:template match="x:PathExpr">
        <xsl:apply-templates select="x:*"/>
    </xsl:template>
    <xsl:template match="x:UnaryExpr">
        <xsl:value-of select="@symbol"/><xsl:apply-templates select="x:*"/>
    </xsl:template>

    <xsl:template match="x:Predicate[count(*)=1]/x:PathExpr[count(*)=1]/x:FilterExpr[count(*)=1]/x:Number" priority="9">
        <xsl:text/>position() = <xsl:value-of select="@i"/>
    </xsl:template>
    <xsl:template match="x:Number"><xsl:value-of select="@i"/></xsl:template>
    <xsl:template match="x:Literal">'<xsl:value-of select="@s"/>'</xsl:template>
    <xsl:template match="x:VariableReference">$<xsl:value-of select="@s1"/></xsl:template>

    <xsl:template match="x:Predicate">[<xsl:apply-templates select="x:*[last()]"/>]</xsl:template>
    <xsl:template match="x:FilterExpr"><xsl:apply-templates select="x:*"/></xsl:template>
    <xsl:template match="x:*[local-name()!='OrExpr' and local-name()!='AndExpr' and local-name()!='UnionExpr' and local-name()!='UnaryExpr'][@symbol]">
        <xsl:apply-templates select="preceding-sibling::x:*[1]"/>
        <xsl:text> </xsl:text><xsl:value-of select="@symbol"/><xsl:text> </xsl:text>
        <xsl:apply-templates select="x:*[last()]"/>
    </xsl:template>

    <xsl:template match="x:XPath">
        <xsl:apply-templates select="x:*[last()]"/>
    </xsl:template>

    <xsl:template match="x:*">
        <xsl:apply-templates select="x:*"/>
    </xsl:template>
</xsl:stylesheet>