<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:r="http://software.in2p3.fr/lavoisier/rendering.xsd">
    <xsl:output method="html"  indent="yes" />
    <xsl:param name="types">column, spline, line, areaspline</xsl:param>
    <xsl:param name="stacking">none</xsl:param>
    <xsl:param name="axis">normal</xsl:param>
    <xsl:param name="rotation">0</xsl:param>
    <xsl:param name="width">0</xsl:param>
    <xsl:param name="height">0</xsl:param>
    <xsl:key name="labelByUnit" match="/r:rendering/r:rows/r:column_labels/r:column_label" use="@unit"/>

    <!-- prevents unwanted text() nodes to be printed -->
    <xsl:template match="/r:rendering">
        <xsl:apply-templates select="r:rows"/>
    </xsl:template>

    <xsl:template match="r:rows">
        <xsl:variable name="highcharts-types" select="str:split($types, ', ')" xmlns:str="http://exslt.org/strings"/>
        <xsl:variable name="highcharts-stacking">
            <xsl:choose>
                <xsl:when test="$stacking='none'">null</xsl:when>
                <xsl:when test="$stacking='normal'">'normal'</xsl:when>
                <xsl:when test="$stacking='percent'">'percent'</xsl:when>
                <xsl:otherwise>
                    <xsl:message terminate="yes">Unsupported chart stacking: <xsl:value-of select="$stacking"/></xsl:message>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:choose>
            <xsl:when test="$axis='normal'"/>
            <xsl:when test="$axis='inverted'"/>
            <xsl:when test="$axis='polar'"/>
            <xsl:otherwise>
                <xsl:message terminate="yes">Unsupported chart axis: <xsl:value-of select="$axis"/></xsl:message>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:variable name="highcharts-units" select="/r:rendering/r:rows/r:column_labels/r:column_label
                                                      [generate-id() = generate-id(key('labelByUnit', @unit)[1])]"/>
        <xsl:variable name="highcharts-colors" xmlns:str="http://exslt.org/strings"
                      select="str:split('4572A7,AA4643,89A54E,80699B,3D96AE,DB843D,92A8CD,A47D7C,B5CA92,4572A7', ',')"/>

<html>
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>
    <xsl:for-each select="/r:rendering/@title">
        <title><xsl:value-of select="."/></title>
    </xsl:for-each>
    <script type="text/javascript" src="/resource/rsc/jquery.js"></script>
    <script type="text/javascript">
$(function () {
    var chart;
    $(document).ready(function() {
        chart = new Highcharts.Chart({
            chart: {
                renderTo: 'container',
        <xsl:if test="$width">
                width: <xsl:value-of select="$width"/>,
        </xsl:if>
        <xsl:if test="$height">
                height: <xsl:value-of select="$height"/>,
        </xsl:if>
                inverted: <xsl:value-of select="string($axis='inverted')"/>,
                polar: <xsl:value-of select="string($axis='polar')"/>
            },
            plotOptions: {
        <xsl:value-of select="$highcharts-types[1]/text()"/>: {
        <xsl:if test="$highcharts-types[1]/text()='column' and $axis='polar'">
                    pointPadding: 0,
                    groupPadding: 0,
        </xsl:if>
                    stacking: <xsl:value-of select="$highcharts-stacking"/>
                }
            },

        <xsl:for-each select="/r:rendering/@title">
            title: {
                text: '<xsl:value-of select="."/>'
            },
        </xsl:for-each>
        <xsl:for-each select="/r:rendering/r:fields">
            subtitle: {
                text: '<xsl:for-each select="r:field">
                <xsl:if test="position() > 1">, </xsl:if>
                <xsl:choose>
                    <xsl:when test="@link">
                        <a href="{@link}"><xsl:value-of select="text()"/></a><xsl:text/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="text()"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:for-each>'
            },
        </xsl:for-each>
            xAxis: {
                categories: [
        <xsl:for-each select="r:row">
            <xsl:if test="position()>1">, </xsl:if>
            <xsl:text>'</xsl:text>
            <xsl:variable name="row" select="."/>
            <xsl:for-each select="/r:rendering/r:rows/r:column_labels/r:column_label[not(@unit)]">
                <xsl:if test="position()>1">_</xsl:if>
                <xsl:value-of select="$row/r:column[count(current()/preceding-sibling::r:column_label)+1]/text()"/>
            </xsl:for-each>
            <xsl:text>'</xsl:text>
        </xsl:for-each>
                ],
                labels: {
                    <xsl:choose>
                        <xsl:when test="$rotation &lt; 0">
                            align: 'right',
                        </xsl:when>
                        <xsl:when test="$rotation &gt; 0">
                            align: 'left',
                        </xsl:when>
                        <xsl:otherwise>
                            align: 'center',
                        </xsl:otherwise>
                    </xsl:choose>
                    rotation: <xsl:value-of select="$rotation"/>
                },
        <xsl:if test="$highcharts-types[1]/text()!='column' and $axis='polar'">
                tickmarkPlacement: 'on',
        </xsl:if>
                title: {
        <xsl:if test="$axis!='polar'">
                    text: '<xsl:for-each select="/r:rendering/r:rows/r:column_labels/r:column_label[not(@unit)]">
            <xsl:if test="position()>1">_</xsl:if>
            <xsl:value-of select="text()"/>
            </xsl:for-each>',
        </xsl:if>
                    align: 'high'
                }
            },
            yAxis: [
        <xsl:for-each select="$highcharts-units">
            <xsl:variable name="unit" select="@unit"/>
            <xsl:variable name="color" select="$highcharts-colors
[position() = count(/r:rendering/r:rows/r:column_labels/r:column_label[@unit=$unit][1]/preceding-sibling::r:column_label)]"/>
            <xsl:if test="position()>1">, </xsl:if>
            {
                <xsl:if test="position()>1">opposite: true,</xsl:if>
                labels: {
                    style: {
                        color: '#<xsl:value-of select="$color"/>'
                    }
                },
                title: {
                    style: {
                        color: '#<xsl:value-of select="$color"/>'
                    },
            <xsl:choose>
            <xsl:when test="$axis='polar'"/>
            <xsl:when test="$stacking='percent'">
                    text: 'percent',
            </xsl:when>
            <xsl:otherwise>
                    text: '<xsl:value-of select="@unit"/>',
            </xsl:otherwise>
        </xsl:choose>
                    align: 'high'
                }
            }
        </xsl:for-each>
            ],
            series: [
        <xsl:for-each select="r:column_labels/r:column_label[@unit]">
            <xsl:variable name="unit" select="@unit"/>
            <xsl:variable name="unitNumber">
                <xsl:for-each select="$highcharts-units">
                    <xsl:if test="@unit = $unit">
                        <xsl:value-of select="position()"/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:if test="position()>1">, </xsl:if>
            {
                name: '<xsl:value-of select="text()"/>',
                type: '<xsl:value-of select="$highcharts-types[position()=$unitNumber]/text()"/>',
                yAxis: <xsl:value-of select="$unitNumber - 1"/>,
                data: [
            <xsl:for-each select="/r:rendering/r:rows/r:row/r:column[count(current()/preceding-sibling::r:column_label)+1]">
                <xsl:if test="position()>1">, </xsl:if>
                <xsl:value-of select="text()"/>
            </xsl:for-each>
                ]
            }
        </xsl:for-each>
            ],

            tooltip: {
                formatter: function() {
                    return ''+ this.series.name +': '+ this.y +'';
                }
            },
            credits: {
                enabled: false
            }
        });
    });
});
    </script>
</head>
<body>
    <script src="/resource/js/highcharts.js"></script>
    <script src="/resource/js/highcharts-more.js"></script>
    <script src="/resource/js/modules/exporting.js"></script>
    <div id="container" style="min-width: 400px; height: 400px; margin: 0 auto"></div>
</body>
</html>

    </xsl:template>
</xsl:stylesheet>
