<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:cfg="http://software.in2p3.fr/lavoisier/config.xsd">
    <xsl:include href="/resource/xsl/head.xsl"/>
    <xsl:include href="/lavoisier/navbar"/>
    <xsl:output method="html" encoding="utf-8" indent="yes"/>

    <xsl:template match="/cfg:view">
        <html>
            <xsl:call-template name="head"/>
            <body data-spy="scroll" data-target=".subnav" data-offset="50">
                <xsl:call-template name="navbar"/>
                <div class="container fluid" style="margin-top:80px">
                    <h1><xsl:value-of select="@name"/></h1>
                    <form id="form" method="get">
                        <xsl:for-each select="cfg:argument">
                            <xsl:value-of select="@name"/>:
                            <xsl:choose>
                                <xsl:when test="contains(@pattern,'|') and not(contains(@pattern, '['))">
                                    <select name="{@name}">
                                        <xsl:call-template name="OPTIONS">
                                            <xsl:with-param name="pattern" select="@pattern"/>
                                            <xsl:with-param name="default" select="text()"/>
                                        </xsl:call-template>
                                    </select>
                                </xsl:when>
                                <xsl:otherwise>
                                    <input type="text" name="{@name}" value="{text()}" class="span4"/>
                                </xsl:otherwise>
                            </xsl:choose>
                            <br/>
                        </xsl:for-each>
                        <hr/>

                        <xsl:for-each select="cfg:processors[contains(cfg:processor/@disabled,'_stop_at_')]">
                            <b>Stop at:</b><br/>
                            <input type="radio" name="_stop_at_" value="0"/>
                            <xsl:value-of select="parent::cfg:view/cfg:connector/@type"/>
                            <br/>
                            <xsl:for-each select="cfg:processor[contains(@disabled,'_stop_at_')]">
                                <input type="radio" name="_stop_at_" value="{position()}"/>
                                <xsl:value-of select="@type"/>(<xsl:value-of select="@match"/>)
                                <br/>
                            </xsl:for-each>
                            <br/>
                        </xsl:for-each>

                        <input type="submit" value="Get data" onclick="get()" class="btn"/>
                        as
                        <select name="accept" id="accept">
                            <xsl:variable name="selected" select="cfg:info/cfg:accept/text()"/>
                            <option value="default">Default rendering</option>
                            <xsl:for-each select="jmx/domain/group/mbean/attribute[@name='Extensions']/value[1]">
                                <option value="{text()}">
                                    <xsl:if test="text() = $selected">
                                        <xsl:attribute name="selected">selected</xsl:attribute>
                                    </xsl:if>
                                    <xsl:value-of select="text()"/>
                                </option>
                            </xsl:for-each>
                        </select>
                        from
                        <input type="text" name="_window_start_" id="_window_start_" class="span1" value="1"/>
                        to
                        <input type="text" name="_window_end_" id="_window_end_" class="span1"/>
                        <input type="hidden" name="_window_" id="_window_"/>
                        <br/>

                        <input type="hidden" name="view" id="view" value="{@name}"/>
                        <input type="submit" value="Breakpoint" onclick="breakpoint()" class="btn"/>
                        on
                        <input type="text" name="_break_at_" id="_break_at_" value="/*/*" class="span4"/>
                        <br/>

                        <input type="submit" value="Get skeleton" onclick="skeleton()" class="btn"/>
                    </form>
                </div>
                <script language="Javascript">
function get() {
    document.getElementById('form').action='/lavoisier/<xsl:value-of select="@name"/>';
    if (document.getElementById('accept').value == 'default') {
        document.getElementById('accept').disabled=true;
    }
    document.getElementById('view').disabled=true;
    common();
}
function breakpoint() {
    document.getElementById('form').action='/lavoisier/breakpoint'+document.getElementById('_break_at_').value;
    document.getElementById('form').target='_blank';
    document.getElementById('accept').disabled=true;
    document.getElementById('view').disabled=false;
    common();
}
function skeleton() {
    document.getElementById('form').action='/lavoisier/skeleton/<xsl:value-of select="@name"/>';
    document.getElementById('accept').disabled=true;
    document.getElementById('view').disabled=true;
    common();
}
function common() {
    document.getElementById('_break_at_').disabled=true;
    document.getElementById('_window_start_').disabled=true;
    document.getElementById('_window_end_').disabled=true;
    if (document.getElementById('_window_end_').value != '') {
        document.getElementById('_window_').value = document.getElementById('_window_start_').value+'-'+document.getElementById('_window_end_').value;
    } else {
        document.getElementById('_window_').disabled=true;
    }
    $("#form input[type=text]").each(function() {
        if (this.value == '') {
            this.disabled = true;
        }
    })
}
                </script>
            </body>
        </html>
    </xsl:template>

    <xsl:template name="OPTIONS">
        <xsl:param name="pattern"/>
        <xsl:param name="default"/>
        <xsl:choose>
            <xsl:when test="contains($pattern, '|')">
                <xsl:variable name="current" select="substring-before($pattern, '|')"/>
                <option value="{$current}">
                    <xsl:if test="$current = $default">
                        <xsl:attribute name="selected">selected</xsl:attribute>
                    </xsl:if>
                    <xsl:value-of select="$current"/>
                </option>
                <xsl:call-template name="OPTIONS">
                    <xsl:with-param name="pattern" select="substring-after($pattern, '|')"/>
                    <xsl:with-param name="default" select="$default"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <option value="{$pattern}">
                    <xsl:if test="$pattern = $default">
                        <xsl:attribute name="selected">selected</xsl:attribute>
                    </xsl:if>
                    <xsl:value-of select="$pattern"/>
                </option>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
</xsl:stylesheet>