<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xalan"
                xmlns:render="http://software.in2p3.fr/lavoisier/rendering.xsd">
    <xsl:output method="xml" indent="yes" xalan:indent-amount="4"/>
    <xsl:strip-space elements="*"/><!-- prevents text() function to ignore the text nodes other than 1st one -->
    <xsl:param name="view">Default rendering</xsl:param>

    <xsl:variable name="lower">abcdefghijklmnopqrstuvwxyz</xsl:variable>
    <xsl:variable name="UPPER">ABCDEFGHIJKLMNOPQRSTUVWXYZ</xsl:variable>

    <!-- rows are in level 2 -->
    <xsl:variable name="map2">map2</xsl:variable>
    <xsl:key name="map2"
             match="/*/*[@* or text()!='' or */text()!='']"
             use="concat('{',namespace-uri(),'}',local-name())"/>

    <xsl:variable name="rows2">rows2</xsl:variable>
    <xsl:key name="rows2"
             match="/*/*[key('map2',concat('{',namespace-uri(),'}',local-name()))[2]]"
             use="concat('{',namespace-uri(),'}',local-name())"/>

    <xsl:variable name="rows3">rows3</xsl:variable>
    <xsl:key name="rows3"
             match="/*/*[key('map2',concat('{',namespace-uri(),'}',local-name()))[2]]/*[text()!='']"
             use="concat('{',namespace-uri(..),'}',local-name(..), '{',namespace-uri(),'}',local-name())"/>

    <xsl:template match="/">
        <render:rendering title="{$view}">
            <xsl:variable name="f2" select="/*/*[not(key($map2,concat('{',namespace-uri(),'}',local-name()))[2])]"/>
            <xsl:variable name="fields" select="/*/@* | /*/text()[.!=''] | $f2/@* | $f2/text()[.!=''] | $f2/*/text()[.!='']"/>
            <xsl:if test="$fields">
                <render:fields>
                    <xsl:apply-templates select="$fields" mode="field"/>
                </render:fields>
            </xsl:if>

            <xsl:variable name="r2" select="/*/*[key($map2,concat('{',namespace-uri(),'}',local-name()))[2]]"/>
            <xsl:choose>
                <!-- rows are in level 2 -->
                <xsl:when test="$r2">
                    <xsl:variable name="c2" select="/*/*[generate-id()=generate-id(key($rows2,
                        concat('{',namespace-uri(),'}',local-name()))[1])]"/>
                    <xsl:variable name="c3" select="/*/*/*[generate-id()=generate-id(key($rows3,
                        concat('{',namespace-uri(..),'}',local-name(..), '{',namespace-uri(),'}',local-name()))[1])]"/>
                    <xsl:variable name="column_names" select="$c2/@* | $c2/text()[.!=''] | $c3/text()[.!='']"/>
                    <render:rows>
                        <render:column_labels>
                            <render:column_label>Position</render:column_label>
                            <xsl:apply-templates select="$column_names" mode="column_label"/>
                        </render:column_labels>
                        <xsl:for-each select="$r2">
                            <render:row>
                                <!-- column Position -->
                                <xsl:choose>
                                    <xsl:when test="*">
                                        <render:column>
                                            <xsl:attribute name="link" xmlns:e="http://software.in2p3.fr/lavoisier/entries.xsd">
                                                <xsl:if test="not(/e:entries)">/lavoisier/<xsl:value-of select="$view"/>/*/</xsl:if>
                                                <xsl:text/>*[<xsl:value-of select="position()"/>]/*?accept=rowcol<xsl:text/>
                                            </xsl:attribute>
                                            <xsl:value-of select="position()"/>
                                        </render:column>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <render:column>
                                            <xsl:value-of select="position()"/>
                                        </render:column>
                                    </xsl:otherwise>
                                </xsl:choose>
                                <!-- other columns -->
                                <xsl:apply-templates select="$column_names" mode="column">
                                    <xsl:with-param name="row" select="."/>
                                </xsl:apply-templates>
                            </render:row>
                        </xsl:for-each>
                    </render:rows>
                </xsl:when>

                <!-- information may be deeper -->
                <xsl:when test="/*/*/*">
                    <render:rows>
                        <render:column_labels>
                            <render:column_label>Position</render:column_label>
                            <render:column_label>Element name</render:column_label>
                        </render:column_labels>
                        <xsl:for-each select="/*/*">
                            <render:row>
                                <!-- column Position -->
                                <xsl:choose>
                                    <xsl:when test="*">
                                        <render:column>
                                            <xsl:attribute name="link" xmlns:e="http://software.in2p3.fr/lavoisier/entries.xsd">
                                                <xsl:if test="not(/e:entries)">/lavoisier/<xsl:value-of select="$view"/>/*/</xsl:if>
                                                <xsl:text/>*[<xsl:value-of select="position()"/>]/*?accept=rowcol<xsl:text/>
                                            </xsl:attribute>
                                            <xsl:value-of select="position()"/>
                                        </render:column>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <render:column>
                                            <xsl:value-of select="position()"/>
                                        </render:column>
                                    </xsl:otherwise>
                                </xsl:choose>
                                <!-- other columns -->
                                <render:column><xsl:value-of select="local-name()"/></render:column>
                            </render:row>
                        </xsl:for-each>
                    </render:rows>
                </xsl:when>
            </xsl:choose>
        </render:rendering>
    </xsl:template>

    <xsl:template match="/*/text() | /*/*/text()" mode="field">
        <render:field label="{local-name(..)}">
            <xsl:value-of select="normalize-space()"/>
        </render:field>
    </xsl:template>
    <xsl:template match="/*/*/*/text()" mode="field">
        <render:field label="{concat(local-name(../..),'_',local-name(..))}">
            <xsl:value-of select="normalize-space()"/>
        </render:field>
    </xsl:template>
    <xsl:template match="/*/@*" mode="field">
        <render:field label="{local-name()}">
            <xsl:value-of select="."/>
        </render:field>
    </xsl:template>
    <xsl:template match="/*/*/@*" mode="field">
        <render:field label="{concat(local-name(..), translate(substring(local-name(),1,1),$lower,$UPPER), substring(local-name(),2))}">
            <xsl:value-of select="."/>
        </render:field>
    </xsl:template>

    <xsl:template match="/*/*/text()" mode="column_label">
        <xsl:apply-templates select="." mode="LABEL">
            <xsl:with-param name="name" select="local-name(..)"/>
        </xsl:apply-templates>
    </xsl:template>
    <xsl:template match="/*/*/*/text()" mode="column_label">
        <xsl:apply-templates select="." mode="LABEL">
            <xsl:with-param name="name" select="concat(local-name(../..),'_',local-name(..))"/>
        </xsl:apply-templates>
    </xsl:template>
    <xsl:template match="/*/*/@*" mode="column_label">
        <xsl:apply-templates select="." mode="LABEL">
            <xsl:with-param name="name" select="concat(local-name(..), translate(substring(local-name(),1,1),$lower,$UPPER), substring(local-name(),2))"/>
        </xsl:apply-templates>
    </xsl:template>
    <xsl:template match="/*/*/*/@*" mode="column_label">
        <xsl:apply-templates select="." mode="LABEL">
            <xsl:with-param name="name" select="concat(local-name(../..),'_',local-name(..), translate(substring(local-name(),1,1),$lower,$UPPER), substring(local-name(),2))"/>
        </xsl:apply-templates>
    </xsl:template>
    <xsl:template match="@* | text()" mode="LABEL">
        <xsl:param name="name"/>
        <render:column_label>
            <xsl:if test=".!='' and translate(.,'0123456789.','')=''">
                <xsl:attribute name="unit"><xsl:value-of select="$name"/></xsl:attribute>
            </xsl:if>
            <xsl:value-of select="$name"/>
        </render:column_label>
    </xsl:template>

    <xsl:template match="/*/*/text()" mode="column">
        <xsl:param name="row"/>
        <xsl:variable name="level2" select="current()/.."/>
        <xsl:variable name="value" select="normalize-space($row
            /self::*[namespace-uri()=namespace-uri($level2) and local-name()=local-name($level2)]
            /text())"/>
        <xsl:choose>
            <xsl:when test="parent::*/*">
                <render:column>
                    <xsl:attribute name="link">
                        <xsl:apply-templates select="." mode="ANCESTORS"/>
                        <xsl:text/>[text()='<xsl:value-of select="$value"/>']/*?accept=rowcol<xsl:text/>
                    </xsl:attribute>
                    <xsl:value-of select="$value"/>
                </render:column>
            </xsl:when>
            <xsl:otherwise>
                <render:column>
                    <xsl:value-of select="$value"/>
                </render:column>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template match="/*/*/*/text()" mode="column">
        <xsl:param name="row"/>
        <xsl:variable name="level2" select="current()/../.."/>
        <xsl:variable name="level3" select="current()/.."/>
        <xsl:variable name="value" select="normalize-space($row
            /self::*[namespace-uri()=namespace-uri($level2) and local-name()=local-name($level2)]
            /*[namespace-uri()=namespace-uri($level3) and local-name()=local-name($level3)]
            /text())"/>
        <render:column>
            <xsl:attribute name="link">
                <xsl:apply-templates select="parent::*" mode="ANCESTORS"/>
                <xsl:text/>[<xsl:apply-templates select="parent::*" mode="SELF"/>/text()='<xsl:value-of select="$value"/>']/*?accept=rowcol<xsl:text/>
            </xsl:attribute>
            <xsl:value-of select="$value"/>
        </render:column>
    </xsl:template>
    <xsl:template match="/*/*/@*" mode="column">
        <xsl:param name="row"/>
        <xsl:variable name="level2" select="current()/.."/>
        <xsl:variable name="attribute" select="current()"/>
        <xsl:variable name="value" select="normalize-space($row
            /self::*[namespace-uri()=namespace-uri($level2) and local-name()=local-name($level2)]
            /@*[local-name()=local-name($attribute)])"/>
        <xsl:choose>
            <xsl:when test="parent::*/*">
                <render:column>
                    <xsl:attribute name="link">
                        <xsl:apply-templates select="." mode="ANCESTORS"/>
                        <xsl:text/>[@<xsl:value-of select="local-name()"/>='<xsl:value-of select="$value"/>']/*?accept=rowcol<xsl:text/>
                    </xsl:attribute>
                    <xsl:value-of select="$value"/>
                </render:column>
            </xsl:when>
            <xsl:otherwise>
                <render:column>
                    <xsl:value-of select="$value"/>
                </render:column>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="* | @* | text()" mode="ANCESTORS">
        <xsl:choose xmlns:e="http://software.in2p3.fr/lavoisier/entries.xsd">
            <xsl:when test="/e:entries">*</xsl:when>
            <xsl:otherwise>
                <xsl:text/>/lavoisier/<xsl:value-of select="$view"/>
                <xsl:for-each select="ancestor::*">/<xsl:apply-templates select="." mode="SELF"/></xsl:for-each>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template match="*" mode="SELF">
        <xsl:choose>
            <xsl:when test="not(parent::*)">*</xsl:when>
            <xsl:when test="boolean(namespace-uri())">*[local-name()='<xsl:value-of select="local-name()"/>']</xsl:when>
            <xsl:otherwise><xsl:value-of select="local-name()"/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
</xsl:stylesheet>