<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:render="http://software.in2p3.fr/lavoisier/rendering.xsd"
                xmlns:x="http://www.w3.org/1999/xhtml"
                xmlns="http://www.w3.org/1999/xhtml"
                exclude-result-prefixes="render x">
    <xsl:output method="xml"/>

    <xsl:variable name="xsl">http://www.w3.org/1999/XSL/Transform</xsl:variable>
    <xsl:variable name="render">http://software.in2p3.fr/lavoisier/rendering.xsd</xsl:variable>
    <xsl:variable name="html">http://www.w3.org/1999/xhtml</xsl:variable>

    <xsl:template match="xsl:stylesheet">
        <xsl:if test="not(descendant::render:rendering | descendant::xsl:element[@name='rendering' and @namespace=$render])">
            <xsl:message terminate="yes">Output of rendering stylesheet must be either HTML, text, or XML conform to
                namespace {<xsl:value-of select="$render"/>}.</xsl:message>
        </xsl:if>
        <xsl:copy><xsl:copy-of select="@*"/><xsl:text>
    </xsl:text>
            <xsl:comment> generated by xslRowColumn-2-xslHtml.xsl </xsl:comment>
            <xsl:element name="include" namespace="{$xsl}">
                <xsl:attribute name="href">/resource/xsl/head.xsl</xsl:attribute>
            </xsl:element>
            <xsl:element name="include" namespace="{$xsl}">
                <xsl:attribute name="href">/lavoisier/navbar</xsl:attribute>
            </xsl:element>
            <!-- create element <output> if it does not exist -->
            <xsl:if test="not(xsl:output)">
                <xsl:element name="output" namespace="{$xsl}">
                    <xsl:attribute name="method">html</xsl:attribute>
                </xsl:element>
            </xsl:if>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="xsl:output">
        <xsl:copy><xsl:copy-of select="@*"/>
            <!-- override attribute @method if it already exists -->
            <xsl:attribute name="method">html</xsl:attribute>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="render:rendering | xsl:element[@name='rendering' and @namespace=$render]">
        <x:html>
            <head>
                <title>
                    <xsl:call-template name="attributes"/>
                </title>
                <style type="text/css">
                    h2            {text-align : center;}
                    TD             {text-align: left;}
                    TD.number      {text-align: right;}
                </style>
                <xsl:element name="call-template" namespace="{$xsl}">
                    <xsl:attribute name="name">head</xsl:attribute>
                </xsl:element>
            </head>
            <body data-spy="scroll" data-target=".subnav" data-offset="50">
                <xsl:element name="call-template" namespace="{$xsl}">
                    <xsl:attribute name="name">navbar</xsl:attribute>
                </xsl:element>
                <div class="container fluid">
                    <div class="page-title">
                        <h2>
                            <xsl:call-template name="attributes"/>
                        </h2>
                    </div>
                    <div style="margin-top:50px">
                        <xsl:call-template name="elements"/>
                    </div>
                </div>
            </body>
        </x:html>
    </xsl:template>
    <xsl:template match="render:rendering/@title">
        <xsl:call-template name="attribute-2-text">
            <xsl:with-param name="string" select="."/>
        </xsl:call-template>
    </xsl:template>
    <xsl:template match="xsl:attribute[@name='title' and (@namespace=$render or not(@namespace))]">
        <xsl:call-template name="elements"/>
    </xsl:template>

    <xsl:template match="render:fields | xsl:element[@name='fields' and @namespace=$render]">
        <ul>
            <xsl:call-template name="all"/>
        </ul>
    </xsl:template>
    <xsl:template match="render:field | xsl:element[@name='field' and @namespace=$render]">
        <li>
            <xsl:apply-templates select="@label | xsl:attribute[@name='label' and (@namespace=$render or not(@namespace))]"/>
            <xsl:choose>
                <xsl:when test="@link | xsl:attribute[@name='link' and (@namespace=$render or not(@namespace))]">
                    <a>
                        <xsl:apply-templates select="@link | xsl:attribute[@name='link' and @namespace=$render]"/>
                        <xsl:call-template name="elements"/>
                    </a>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:call-template name="elements"/>
                </xsl:otherwise>
            </xsl:choose>
        </li>
    </xsl:template>
    <xsl:template match="render:field/@label">
        <b>
            <xsl:call-template name="attribute-2-text">
                <xsl:with-param name="string" select="."/>
            </xsl:call-template>
            <xsl:text>: </xsl:text>
        </b>
    </xsl:template>
    <xsl:template match="xsl:attribute[@name='label' and (@namespace=$render or not(@namespace))]">
        <b>
            <xsl:call-template name="elements"/>
            <xsl:text>: </xsl:text>
        </b>
    </xsl:template>

    <xsl:template match="render:rows | xsl:element[@name='rows' and @namespace=$render]">
        <table class="table table-bordered table-striped table-sorted table-hover">
            <xsl:call-template name="all"/>
        </table>
    </xsl:template>

    <xsl:template match="render:column_labels | xsl:element[@name='column_labels' and @namespace=$render]">
        <thead>
            <tr>
                <xsl:call-template name="all"/>
            </tr>
        </thead>
    </xsl:template>
    <xsl:template match="render:column_label | xsl:element[@name='column_label' and @namespace=$render]">
        <th>
            <xsl:call-template name="all"/>
            <xsl:apply-templates mode="unit" select="@unit | descendant::xsl:attribute[@name='unit' and (@namespace=$render or not(@namespace))]"/>
        </th>
    </xsl:template>
    <xsl:template match="@unit | xsl:attribute[@name='unit' and (@namespace=$render or not(@namespace))]"/>
    <xsl:template match="@unit" mode="unit">
        <xsl:text> (</xsl:text>
        <xsl:value-of select="."/>
        <xsl:text>)</xsl:text>
    </xsl:template>
    <xsl:template match="xsl:attribute[@name='unit' and (@namespace=$render or not(@namespace))]" mode="unit">
        <xsl:text> (</xsl:text>
        <xsl:call-template name="all"/>
        <xsl:text>)</xsl:text>
    </xsl:template>

    <xsl:template match="render:row | xsl:element[@name='row' and @namespace=$render]">

        <tr>
            <xsl:call-template name="all"/>
        </tr>

    </xsl:template>
    <xsl:template match="render:column | xsl:element[@name='column' and @namespace=$render]">
        <td>
            <xsl:choose>
                <xsl:when test="@link | xsl:attribute[@name='link' and (@namespace=$render or not(@namespace))]">
                    <a>
                        <xsl:call-template name="all"/>
                    </a>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:call-template name="elements"/>
                </xsl:otherwise>
            </xsl:choose>
        </td>
    </xsl:template>
    <xsl:template match="render:field/@link | render:column/@link">
        <xsl:attribute name="href">
            <xsl:value-of select="."/>
        </xsl:attribute>
    </xsl:template>
    <xsl:template match="xsl:attribute[@name='link' and (@namespace=$render or not(@namespace))]">
        <xsl:element name="attribute" namespace="{$xsl}">
            <xsl:attribute name="name">href</xsl:attribute>
            <xsl:call-template name="elements"/>
        </xsl:element>
    </xsl:template>

    <!-- error cases -->
    <xsl:template match="render:* | render:*/@*" priority="-1">
        <xsl:message terminate="yes">Unsupported node: <xsl:value-of select="local-name()"/></xsl:message>
    </xsl:template>
    <xsl:template match="xsl:element[@namespace=$render] | xsl:attribute[@namespace=$render]" priority="-1">
        <xsl:message terminate="yes">Unsupported node: <xsl:value-of select="@name"/></xsl:message>
    </xsl:template>

    <!-- default cases -->
    <xsl:template match="*" priority="-2">
        <xsl:copy>
            <xsl:apply-templates select="@*|*|text()|comment()"/>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="@*|text()|comment()" priority="-2">
        <xsl:copy-of select="."/>
    </xsl:template>

    <!-- functions -->
    <xsl:template name="all">
        <xsl:apply-templates select="@*[not(parent::xsl:element)] | * | text() | comment()"/>
    </xsl:template>
    <xsl:template name="attributes">
        <xsl:apply-templates select="@*[not(parent::xsl:element)] | xsl:attribute"/>
    </xsl:template>
    <xsl:template name="elements">
        <xsl:apply-templates select="*[not(self::xsl:attribute)] | text() | comment()"/>
    </xsl:template>
    <xsl:template name="attribute-2-text">
        <xsl:param name="string"/>
        <xsl:choose>
            <xsl:when test="contains($string, '{')">
                <xsl:value-of select="substring-before($string, '{')"/>
                <xsl:variable name="remains" select="substring-after($string, '{')"/>
                <xsl:element name="value-of" namespace="{$xsl}">
                    <xsl:attribute name="select">
                        <xsl:value-of select="substring-before($remains, '}')"/>
                    </xsl:attribute>
                </xsl:element>
                <!-- recurse -->
                <xsl:call-template name="attribute-2-text">
                    <xsl:with-param name="string" select="substring-after($remains, '}')"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$string"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
</xsl:stylesheet>